/**
 * HttpManager.java 2014-4-24
 * 
 * 天津云翔联动科技有限公司(c) 1995 - 2014 。
 * http://www.soaring-cloud.com.cn
 *
 */
package com.soaringcloud.library.network;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Properties;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.http.Header;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.HttpVersion;
import org.apache.http.ParseException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.GzipDecompressingEntity;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.conn.params.ConnPerRouteBean;
import org.apache.http.conn.params.ConnRouteParams;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.telephony.TelephonyManager;
import android.util.Log;

import com.soaringcloud.library.common.Constant;
import com.soaringcloud.library.exception.BadServerException;
import com.soaringcloud.library.exception.ErrorMessage;
import com.soaringcloud.library.exception.NetworkErrorException;
import com.soaringcloud.library.exception.TimeOutException;
import com.soaringcloud.library.exception.UnauthorizedException;
import com.soaringcloud.library.util.NetworkUtil;
import com.soaringcloud.library.util.PropertyUtil;

/**
 * 
 * <b>HttpManager。</b>
 * <p><b>详细说明：</b></p>
 * <!-- 在此添加详细说明 -->
 * 无。
 * <p><b>修改列表：</b></p>
 * <table width="100%" cellSpacing=1 cellPadding=3 border=1>
 * <tr bgcolor="#CCCCFF"><td>序号</td><td>作者</td><td>修改日期</td><td>修改内容</td></tr>
 * <!-- 在此添加修改列表，参考第一行内容 -->
 * <tr><td>1</td><td>Wancheng</td><td>2014-4-25 下午2:43:14</td><td>建立类型</td></tr>
 * 
 * </table>
 * @version 1.0
 * @author Wancheng
 * @since 1.0
 */
public class HttpManager {

	/**
	 * 日志输出用TAG
	 */
	private static final String TAG = HttpManager.class.getName();
	
	private static HttpClient httpClient;
	
	private static HttpManager manager;
	
	/**
	 * properties配置文件名称（全称，含后缀）
	 */
	public static String CONFIG_FILE_NAME = null;
	
	/**
	 * 
	 * <b>构造方法。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 私有构造方法
	 */
	private HttpManager() {
		
	}
	
	/**
	 * 
	 * <b>getInstance。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 返回本类的实例
	 * @return
	 */
	public static HttpManager getInstance(){
		if(null == manager){
			synchronized(HttpManager.class){
				if (null == manager)
					manager = new HttpManager();
					Log.i(TAG, "Init HttpManager....");
			}
		}
		return manager;
	}

	/**
	 * 默认初始化
	 */
	static {
		long beginTime = System.currentTimeMillis();
		HttpParams httpParams = new BasicHttpParams();
		Properties properties = PropertyUtil.getInstance().getProperties(CONFIG_FILE_NAME);
		if(null != properties){
			try{
				initFromConfig(httpParams,properties,null);
			}catch (Exception e) {
				// init use default settings
				httpParams = new BasicHttpParams();
				defaultInit(httpParams,null);
			}
		}else{
			defaultInit(httpParams,null);
		}
		SchemeRegistry schemeRegistry = new SchemeRegistry();
		schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		try {
			KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
			trustStore.load(null, null);
			SSLSocketFactory sf = new MySSLSocketFactory(trustStore);
			sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			schemeRegistry.register(new Scheme("https", sf, 443));
		} catch (Exception ex) {
			// do nothing, just keep not crash
		}
		ClientConnectionManager manager = new ThreadSafeClientConnManager(httpParams, schemeRegistry);
		httpClient = new DefaultHttpClient(manager, httpParams);
		long endTime = System.currentTimeMillis();
		Log.d(TAG, "Init Cost Total [" + (endTime - beginTime) + "]ms");
	}
	
	/**
	 * 
	 * <b>initFromConfig。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 从配置文件（CONFIG_FILE_NAME）初始化参数
	 * @param httpParams
	 * @param properties
	 * @throws NumberFormatException
	 */
	private static void initFromConfig(HttpParams httpParams, Properties properties, IUserAgent userAgent) throws NumberFormatException{
		ConnManagerParams.setTimeout(httpParams, Long.parseLong(properties.getProperty("GET_CONN_POOL_TIMEOUT")));
		ConnManagerParams.setMaxConnectionsPerRoute(httpParams, new ConnPerRouteBean(Integer.parseInt(properties.getProperty("MAX_CONNECTIONS_PER_ROUTE"))));
		ConnManagerParams.setMaxTotalConnections(httpParams, Integer.parseInt(properties.getProperty("MAX_TOTAL_CONNECTIONS")));
		HttpClientParams.setRedirecting(httpParams, Boolean.parseBoolean(properties.getProperty("DEFAULT_REDIRECTING")));
		HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setUserAgent(httpParams, null == userAgent ? "null" : userAgent.generateUA());
		HttpProtocolParams.setContentCharset(httpParams, properties.getProperty("CHARSET") == null || Constant.BLANK_STR.equals(properties.getProperty("CHARSET").trim()) ? "UTF-8" : properties.getProperty("CHARSET"));
		HttpConnectionParams.setStaleCheckingEnabled(httpParams, Boolean.parseBoolean("STALE_CHECKING_ENABLED"));
		HttpConnectionParams.setSoTimeout(httpParams, Integer.parseInt(properties.getProperty("DEFAULT_SOCKET_TIMEOUT")));
		HttpConnectionParams.setConnectionTimeout(httpParams, Integer.parseInt(properties.getProperty("HTTP_CONN_TIMEOUT")));
		HttpConnectionParams.setTcpNoDelay(httpParams, Boolean.parseBoolean(properties.getProperty("TCP_NO_DELAY")));
		HttpConnectionParams.setSocketBufferSize(httpParams, Integer.parseInt(properties.getProperty("DEFAULT_SOCKET_BUFFER_SIZE")));
		Log.d(TAG, "Init From User Config...");
	}
	
	/**
	 * 
	 * <b>defaultInit。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 默认的初始化参数
	 * @param httpParams
	 */
	private static void defaultInit(HttpParams httpParams, IUserAgent userAgent){
		ConnManagerParams.setTimeout(httpParams, Constant.GET_CONN_POOL_TIMEOUT);
		ConnManagerParams.setMaxConnectionsPerRoute(httpParams, new ConnPerRouteBean(Constant.MAX_CONNECTIONS_PER_ROUTE));
		ConnManagerParams.setMaxTotalConnections(httpParams, Constant.MAX_TOTAL_CONNECTIONS);
		HttpClientParams.setRedirecting(httpParams, Constant.DEFAULT_REDIRECTING);
		HttpProtocolParams.setVersion(httpParams, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setUserAgent(httpParams, null == userAgent ? "null" : userAgent.generateUA());
		HttpProtocolParams.setContentCharset(httpParams, Constant.CHARSET);
		HttpConnectionParams.setStaleCheckingEnabled(httpParams, Constant.STALE_CHECKING_ENABLED);
		HttpConnectionParams.setSoTimeout(httpParams, Constant.DEFAULT_SOCKET_TIMEOUT);
		HttpConnectionParams.setConnectionTimeout(httpParams, Constant.HTTP_CONN_TIMEOUT);
		HttpConnectionParams.setTcpNoDelay(httpParams, Constant.TCP_NO_DELAY);
		HttpConnectionParams.setSocketBufferSize(httpParams, Constant.DEFAULT_SOCKET_BUFFER_SIZE);
		Log.d(TAG, "Init From Default Settings...");
	}
	
	/**
	 * 
	 * <b>reloadUserSettings。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据指定的配置文件重新加载并生成HttpClient
	 * @param configFileName
	 */
	public static void reloadUserSettings(String configFileName, IUserAgent userAgent){
		CONFIG_FILE_NAME = configFileName;
		HttpParams httpParams = new BasicHttpParams();
		Properties properties = PropertyUtil.getInstance().getProperties(CONFIG_FILE_NAME);
		if(null != properties){
			try{
				initFromConfig(httpParams,properties,userAgent);
			}catch (Exception e) {
				Log.e(TAG, "Reload configuration error, Exception is [" + e.toString() + "]");
				return;
			}
		}else{
			Log.e(TAG, "Reload configuration fail, Properties is null");
			return;
		}
		SchemeRegistry schemeRegistry = new SchemeRegistry();
		schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		try {
			KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
			trustStore.load(null, null);
			SSLSocketFactory sf = new MySSLSocketFactory(trustStore);
			sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			schemeRegistry.register(new Scheme("https", sf, 443));
		} catch (Exception ex) {
			// do nothing, just keep not crash
		}
		ClientConnectionManager manager = new ThreadSafeClientConnManager(httpParams, schemeRegistry);
		synchronized (HttpManager.class) {
			httpClient = new DefaultHttpClient(manager, httpParams);
		}
	}
	
	/**
	 * 
	 * <b>reloadDefaultSettings。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 使用默认参数重新加载并生成HttpClient
	 */
	public static void reloadDefaultSettings(IUserAgent userAgent){
		HttpParams httpParams = new BasicHttpParams();
		defaultInit(httpParams, userAgent);
		SchemeRegistry schemeRegistry = new SchemeRegistry();
		schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		try {
			KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
			trustStore.load(null, null);
			SSLSocketFactory sf = new MySSLSocketFactory(trustStore);
			sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
			schemeRegistry.register(new Scheme("https", sf, 443));
		} catch (Exception ex) {
			// do nothing, just keep not crash
		}
		ClientConnectionManager manager = new ThreadSafeClientConnManager(httpParams, schemeRegistry);
		synchronized (HttpManager.class) {
			httpClient = new DefaultHttpClient(manager, httpParams);
		}
	}

	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpHead进行Http请求并返回HttpResponse
	 * @param head
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(HttpHead head) throws IOException {
		return httpClient.execute(head);
	}
	
	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpHost和HttpGet进行Http请求并返回HttpResponse
	 * @param host
	 * @param get
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(HttpHost host, HttpGet get) throws IOException {
		return httpClient.execute(host, get);
	}

	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpGet进行Http请求并返回HttpResponse（会根据网络状态切换WAP代理）
	 * @param context
	 * @param get
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(Context context, HttpGet get) throws IOException {
		if (!NetworkUtil.isWifiAvailable(context) && NetworkUtil.isWapNetwork(context)) {
			setWapProxy(context);
			return httpClient.execute(get);
		}

		final HttpHost host = (HttpHost) httpClient.getParams().getParameter(ConnRouteParams.DEFAULT_PROXY);
		if (host != null) {
			httpClient.getParams().removeParameter(ConnRouteParams.DEFAULT_PROXY);
		}

		return httpClient.execute(get);
	}

	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpUriRequest进行Http请求并返回HttpResponse（会根据网络状态切换WAP代理）
	 * @param context
	 * @param post
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(Context context, HttpUriRequest post) throws IOException {
		if (!NetworkUtil.isWifiAvailable(context) && NetworkUtil.isWapNetwork(context)) {
			setWapProxy(context);
		}
		return httpClient.execute(post);
	}

	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpPost进行Http请求并返回HttpResponse（会根据网络状态切换WAP代理）
	 * @param context
	 * @param post
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(Context context, HttpPost post) throws IOException {
		if (!NetworkUtil.isWifiAvailable(context) && NetworkUtil.isWapNetwork(context)) {
			setWapProxy(context);
			return httpClient.execute(post);
		}

		final HttpHost host = (HttpHost) httpClient.getParams().getParameter(ConnRouteParams.DEFAULT_PROXY);
		if (host != null) {
			httpClient.getParams().removeParameter(ConnRouteParams.DEFAULT_PROXY);
		}
		return httpClient.execute(post);
	}

	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据HttpUriRequest进行Http请求并返回HttpResponse
	 * @param post
	 * @return
	 * @throws IOException
	 */
	public HttpResponse execute(HttpUriRequest post) throws IOException {
		return httpClient.execute(post);
	}
	
	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据指定的URL和参数(BasicNameValuePair)进行Http请求并返回HttpResponse
	 * @param headers
	 * @param url
	 * @param paramArray
	 * @return
	 * @throws IOException
	 * @throws TimeOutException
	 * @throws BadServerException
	 * @throws NetworkErrorException
	 * @throws UnauthorizedException
	 */
	public HttpResponse execute(Header[] headers, String url, ArrayList<BasicNameValuePair> paramArray) throws IOException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpUriRequest post = getPostRequest(headers, url);
		setEntity(paramArray, post);
		return checkResponseStatus(httpClient.execute(post));
	}
	
	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据指定的URL和参数(JSONObject)进行Http请求并返回HttpResponse
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws TimeOutException
	 * @throws BadServerException
	 * @throws NetworkErrorException
	 * @throws UnauthorizedException
	 */
	public HttpResponse execute(Header[] headers, String url, JSONObject params) throws IOException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpUriRequest post = getPostRequest(headers, url);
		setEntity(params, post);
		return checkResponseStatus(httpClient.execute(post));
	}
	
	/**
	 * 
	 * <b>execute。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 根据指定的URL和参数(JSONArray)进行Http请求并返回HttpResponse
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public HttpResponse execute(Header[] headers, String url, JSONArray params) throws IOException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpUriRequest post = getPostRequest(headers, url);
		setEntity(params, post);
		return checkResponseStatus(httpClient.execute(post));
	}
	
	/**
	 * 
	 * <b>getResultArray。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON数组
	 * @param headers
	 * @param url
	 * @param paramArray
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONArray getResultArray(Header[] headers, String url, ArrayList<BasicNameValuePair> paramArray) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, paramArray);
		String responseStr = getResponseString(httpResponse);
		JSONArray result = new JSONArray(responseStr);
		return result;
	}
	
	/**
	 * 
	 * <b>getResultArray。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON数组
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONArray getResultArray(Header[] headers, String url, JSONObject params) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, params);
		String responseStr = EntityUtils.toString(httpResponse.getEntity()); 
		JSONArray result = new JSONArray(responseStr);
		return result;
	}
	
	/**
	 * 
	 * <b>getResultArray。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON数组
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONArray getResultArray(Header[] headers, String url, JSONArray params) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, params);
		String responseStr = getResponseString(httpResponse);
		JSONArray result = new JSONArray(responseStr);
		return result;
	}

	/**
	 * 
	 * <b>getSingleResult。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON对象
	 * @param headers
	 * @param url
	 * @param paramArray
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONObject getSingleResult(Header[] headers, String url, ArrayList<BasicNameValuePair> paramArray) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, paramArray);
		String responseStr = getResponseString(httpResponse);
		JSONObject result = new JSONObject(responseStr);
		return result;
	}
	
	/**
	 * 
	 * <b>getSingleResult。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON对象
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONObject getSingleResult(Header[] headers, String url, JSONObject params) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, params);
		String responseStr = getResponseString(httpResponse);
		JSONObject result = new JSONObject(responseStr);
		return result;
	}
	
	/**
	 * 
	 * <b>getSingleResult。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 获取Response返回的JSON对象
	 * @param headers
	 * @param url
	 * @param params
	 * @return
	 * @throws IOException
	 * @throws JSONException
	 * @throws NetworkErrorException 
	 * @throws BadServerException 
	 * @throws TimeOutException 
	 */
	public JSONObject getSingleResult(Header[] headers, String url, JSONArray params) throws IOException, JSONException, TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException {
		HttpResponse httpResponse = execute(headers, url, params);
		String responseStr = getResponseString(httpResponse);
		JSONObject result = new JSONObject(responseStr);
		return result;
	}
	
	/**
	 * 
	 * <b>getPostRequest。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 对Request添加指定的头信息
	 * @param headers
	 * @param url
	 * @return
	 */
	public HttpUriRequest getPostRequest(Header[] headers, String url){
    	HttpUriRequest httpRequest = new HttpPost(url);
    	if (null != headers) httpRequest.setHeaders(headers);
		httpRequest.setParams(httpClient.getParams());
		return httpRequest;
    }
	
	/**
	 * 
	 * <b>getResponseString。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 对Response进行解析，判断是否进行了GZip压缩，并输出解压结果
	 * @param httpResponse
	 * @return
	 * @throws ParseException
	 * @throws IOException
	 */
	private String getResponseString(HttpResponse httpResponse) throws ParseException, IOException{
		if(null == httpResponse) return null;
		if (null != httpResponse.getEntity().getContentEncoding() && httpResponse.getEntity().getContentEncoding().getValue().contains("gzip")) {
			return EntityUtils.toString(new GzipDecompressingEntity(httpResponse.getEntity())); 
		} else {
			return EntityUtils.toString(httpResponse.getEntity()); 
		}
	}
	
	/**
	 * 
	 * <b>setEntity。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 对request添加entity信息（添加请求参数）
	 * @param paramArray
	 * @param httpRequest
	 */
	private void setEntity(ArrayList<BasicNameValuePair> paramArray, HttpUriRequest httpRequest) {
		if (null != paramArray && paramArray.size() > 0) {
			try {
				UrlEncodedFormEntity entity = new UrlEncodedFormEntity(paramArray, "UTF-8");
				((HttpPost) httpRequest).setEntity(entity);
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
				Log.e(TAG, e.toString());
			}
		}
	}
	
	/**
	 * 
	 * <b>setEntity。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 对request添加entity信息（添加请求参数）
	 * @param params
	 * @param httpRequest
	 */
	private void setEntity(JSONObject params, HttpUriRequest httpRequest){
		if(null!= params && params.keys().hasNext()){
			try {
				StringEntity entity = new StringEntity(params.toString(), "UTF-8");
				entity.setContentType("application/json");
				((HttpPost) httpRequest).setEntity(entity);
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
				Log.e(TAG, e.toString());
			}
		}
	}
	
	/**
	 * 
	 * <b>setEntity。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 对request添加entity信息（添加请求参数）
	 * @param params
	 * @param httpRequest
	 */
	//TODO need test JSONArray param
	private void setEntity(JSONArray params, HttpUriRequest httpRequest){
		if(null!= params && params.length() > 0){
			try {
				StringEntity entity = new StringEntity(params.toString(), "UTF-8");
				entity.setContentType("application/json");
				((HttpPost) httpRequest).setEntity(entity);
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
				Log.e(TAG, e.toString());
			}
		}
	}
	
	/**
	 * 
	 * <b>closeConnection。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 关闭连接
	 */
	public void closeConnection() {
        if(null != httpClient && null != httpClient.getConnectionManager()) {
            httpClient.getConnectionManager().shutdown();
            httpClient = null;
            Log.i(TAG, "Close Connection...");
        }
    }
	
	/**
	 * 
	 * <b>checkResponseStatus。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 检查Response的状态
	 * @param httpResponse
	 * @return
	 */
	private HttpResponse checkResponseStatus(HttpResponse httpResponse) throws TimeOutException, BadServerException, NetworkErrorException, UnauthorizedException{
		//连接超时
		//服务器错误
		//网络错误
		if(null != httpResponse){
			switch(httpResponse.getStatusLine().getStatusCode()){
				case HttpStatus.SC_REQUEST_TIMEOUT:
					throw new TimeOutException(ErrorMessage.REQUEST_TIME_OUT);
				case HttpStatus.SC_UNAUTHORIZED:
					throw new UnauthorizedException(ErrorMessage.UNAUTHORIZED);
				case HttpStatus.SC_NOT_FOUND:
				case HttpStatus.SC_INTERNAL_SERVER_ERROR:
					throw new BadServerException(ErrorMessage.SERVER_ERROR);
				case HttpStatus.SC_OK:
					return httpResponse;
				default:
					throw new NetworkErrorException(ErrorMessage.NETWORK_ERROR);
			}
		}else{
			throw new NetworkErrorException();
		}
	}

	/**
	 * 
	 * <b>setWapProxy。</b>  
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * 设置WAP代理
	 * @param context
	 */
	private void setWapProxy(Context context) {
		HttpHost host = (HttpHost) httpClient.getParams().getParameter(ConnRouteParams.DEFAULT_PROXY);
		if (host == null) {
			TelephonyManager telMgr = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
//			telMgr.getNetworkCountryIso();//获取电信网络国别
//			telMgr.getPhoneType();//获得通信类型
//			telMgr.getNetworkType();//获得网络类型 
			int phoneType = telMgr.getPhoneType();//获得通信类型
			HttpHost proxy = null;
			switch(phoneType){
				case TelephonyManager.PHONE_TYPE_GSM:
					proxy = new HttpHost(NetworkUtil.CMWAP_PROXY, NetworkUtil.CMWAP_PROXY_PORT);
					break;
				case TelephonyManager.PHONE_TYPE_CDMA:
					proxy = new HttpHost(NetworkUtil.CDMA_PROXY, NetworkUtil.CDMA_PROXY_PORT);
					break;
				default:
					proxy = new HttpHost(NetworkUtil.CMWAP_PROXY, NetworkUtil.CMWAP_PROXY_PORT);
			}
			httpClient.getParams().setParameter(ConnRouteParams.DEFAULT_PROXY, proxy);
		}
	}
	
	/**
	 * 
	 * <b>MySSLSocketFactory。</b>
	 * <p><b>详细说明：</b></p>
	 * <!-- 在此添加详细说明 -->
	 * SSL链接工厂类
	 * <p><b>修改列表：</b></p>
	 * <table width="100%" cellSpacing=1 cellPadding=3 border=1>
	 * <tr bgcolor="#CCCCFF"><td>序号</td><td>作者</td><td>修改日期</td><td>修改内容</td></tr>
	 * <!-- 在此添加修改列表，参考第一行内容 -->
	 * <tr><td>1</td><td>Wancheng</td><td>2014-4-28 下午3:11:59</td><td>建立类型</td></tr>
	 * 
	 * </table>
	 * @version 1.0
	 * @author Wancheng
	 * @since 1.0
	 */
	private static class MySSLSocketFactory
			extends SSLSocketFactory {

		/**
		 * 
		 */
		SSLContext sslContext = SSLContext.getInstance("TLS");

		/**
		 * 
		 * <b>构造方法。</b>  
		 * <p><b>详细说明：</b></p>
		 * <!-- 在此添加详细说明 -->
		 * 无。
		 * @param truststore
		 * @throws NoSuchAlgorithmException
		 * @throws KeyManagementException
		 * @throws KeyStoreException
		 * @throws UnrecoverableKeyException
		 */
		public MySSLSocketFactory(KeyStore truststore) throws NoSuchAlgorithmException, KeyManagementException, KeyStoreException, UnrecoverableKeyException {
			super(truststore);

			TrustManager tm = new X509TrustManager() {

				/**
				 * 
				 * <b>checkClientTrusted。</b>  
				 * @see javax.net.ssl.X509TrustManager#checkClientTrusted(java.security.cert.X509Certificate[], java.lang.String)
				 */
				@Override
				public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
				}

				/**
				 * 
				 * <b>checkServerTrusted。</b>  
				 * @see javax.net.ssl.X509TrustManager#checkServerTrusted(java.security.cert.X509Certificate[], java.lang.String)
				 */
				@Override
				public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
				}

				/**
				 * 
				 * <b>getAcceptedIssuers。</b>  
				 * @see javax.net.ssl.X509TrustManager#getAcceptedIssuers()
				 */
				@Override
				public X509Certificate[] getAcceptedIssuers() {
					return null;
				}
			};

			sslContext.init(null, new TrustManager[] { tm }, null);
		}

		/**
		 * 
		 * <b>createSocket。</b>  
		 * @see org.apache.http.conn.ssl.SSLSocketFactory#createSocket(java.net.Socket, java.lang.String, int, boolean)
		 */
		@Override
		public Socket createSocket(Socket socket, String host, int port, boolean autoClose) throws IOException, UnknownHostException {
			return sslContext.getSocketFactory().createSocket(socket, host, port, autoClose);
		}

		/**
		 * 
		 * <b>createSocket。</b>  
		 * @see org.apache.http.conn.ssl.SSLSocketFactory#createSocket()
		 */
		@Override
		public Socket createSocket() throws IOException {
			return sslContext.getSocketFactory().createSocket();
		}
	}
}
