/**
 * NetworkUtil.java 2014-4-25
 * 
 * 天津云翔联动科技有限公司(c) 1995 - 2014 。
 * http://www.soaring-cloud.com.cn
 *
 */
package com.soaringcloud.library.util;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;


/**
 * <b>NetworkUtil。</b>
 * <p><b>详细说明：</b></p>
 * <!-- 在此添加详细说明 -->
 * 无。
 * <p><b>修改列表：</b></p>
 * <table width="100%" cellSpacing=1 cellPadding=3 border=1>
 * <tr bgcolor="#CCCCFF"><td>序号</td><td>作者</td><td>修改日期</td><td>修改内容</td></tr>
 * <!-- 在此添加修改列表，参考第一行内容 -->
 * <tr><td>1</td><td>Wancheng</td><td>2014-4-25 下午3:42:37</td><td>建立类型</td></tr>
 * 
 * </table>
 * @version 1.0
 * @author Wancheng
 * @since 1.0
 */
public class NetworkUtil {
	
	public static final String CMWAP_PROXY = "10.0.0.172";
    public static final int CMWAP_PROXY_PORT = 80;
    public static final String CDMA_PROXY = "10.0.0.200";
    public static final int CDMA_PROXY_PORT = 80;
    
    /**
     * TODO: the same as Constants, need clear later.
     * 
     * client error code [0, 200)
     * public error code [0, 99]
     * module error code [100, 199]
     * 0 is reserved.
     */
    public static final int    ERROR_NETWORK_PROTOCOL_EXCEPTION          = 1;
    public static final int    ERROR_NETWORK_IO_EXCEPTION                = 2;
    public static final int    ERROR_NETWORK_PARSE_EXCEPTION             = 3;
    public static final int    ERROR_NETWORK_CONNECTION_TIMEOUT          = 4;
    public static final int    ERROR_NETWORK_SOCKET_TIMEOUT              = 5;
    public static final int    ERROR_SERVER_RESPONSE_PARSE_EXCEPTION     = 6;
    
	// wifi status
	/**
	 * Wifi is not enable
	 */
	public static final int WIFI_STATUS_DISABLE = -1;

	/**
	 * Wifi is enable, but not connect to any network
	 */
	public static final int WIFI_STATUS_ENABLE = 0;

	/**
	 * Wifi is enable and connect to some network except CMCC wifi
	 */
	public static final int WIFI_STATUS_CONNECT = 1;

	/**
	 * Wifi is enable and connect with CMCC wifi, but may not verified
	 */
	public static final int WIFI_STATUS_CONNECT_CMCC = 2;
	
	/**
	 * Wifi is enable and connect with ChinaUnicom wifi, but may not verified
	 */
	public static final int WIFI_STATUS_CONNECT_CHINAUNICOM = 3;
	
	/**
	 * Wifi is enable and connect with ChinaNet wifi, but may not verified
	 */
	public static final int WIFI_STATUS_CONNECT_CHINANET = 4;

	
	
	public static final String SSID_CMCC = "CMCC";
	public static final String SSID_CMCC_EDU = "CMCC-EDU";
	public static final String SSID_CHINAUNICOM = "ChinaUnicom";
	public static final String SSID_CHINAUNICOM_U = "ChinaUnicom-U";
	public static final String SSID_CHINANET = "ChinaNet";
	public static final String SSID_CHINANET_EDU = "ChinaNet-EDU";
	
	public static final String REGX_MAIL = "^[\\-\\.\\w]+@[\\.\\-\\w]+(\\.\\w+)+$";
	public static final String REGX_HTTP = "^[A-Za-z][A-Za-z0-9+.-]{1,120}:[A-Za-z0-9/](([A-Za-z0-9$_.+!*,;/?:@&~=-])|%[A-Fa-f0-9]{2}){1,333}(#([a-zA-Z0-9][a-zA-Z0-9$_.+!*,;/?:@&~=%-]{0,1000}))?$";
	public static final String REGX_PHONE = "(1)\\d{10}";

	
	public static int getWifiStatus(Context inContext) {
		Context context = inContext.getApplicationContext();
		WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		if (wifi.isWifiEnabled()) {
			ConnectivityManager connect = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo.State state = connect.getNetworkInfo(ConnectivityManager.TYPE_WIFI).getState();
			if (state == NetworkInfo.State.CONNECTED) {
				WifiInfo info = wifi.getConnectionInfo();
				String ssid = info.getSSID();
				if (info != null && info.getSSID() != null && (ssid.equals(SSID_CMCC) || ssid.equals(SSID_CMCC_EDU))) {
					return WIFI_STATUS_CONNECT_CMCC; // connect to CMCC, but may be not verified
				} else if(info != null && info.getSSID() != null && (ssid.equals(SSID_CHINAUNICOM) || ssid.equals(SSID_CHINAUNICOM_U))){
					return WIFI_STATUS_CONNECT_CHINAUNICOM; // connect to ChinaUnicom, but may be not verified
				} else if(info != null && info.getSSID() != null && (ssid.equals(SSID_CHINANET) || ssid.equals(SSID_CHINANET_EDU))){
					return WIFI_STATUS_CONNECT_CHINANET; // connect to ChinaNet, but may be not verified
				} else {
					return WIFI_STATUS_CONNECT; // connect to wifi
				}
			} else {
				return WIFI_STATUS_ENABLE; // wifi is enable, but no wifi connect
			}
		} else {
			return WIFI_STATUS_DISABLE; // wifi is not enable
		}
	}
	
	public static boolean isWifiAvailable(Context inContext) {
		Context context = inContext.getApplicationContext();
		ConnectivityManager connectivity = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectivity != null) {
			NetworkInfo[] info = connectivity.getAllNetworkInfo();
			if (info != null) {
				for (int i = 0; i < info.length; i++) {
					if (info[i].getTypeName().equals("WIFI") && info[i].isConnected()) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	public static boolean isWapNetwork(Context inContext) {
		Context context = inContext.getApplicationContext();
        int wifiStatus = getWifiStatus(context);
        if(wifiStatus == WIFI_STATUS_DISABLE || wifiStatus == WIFI_STATUS_ENABLE) {
            ConnectivityManager conManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
            NetworkInfo ni = conManager.getActiveNetworkInfo();
            if(ni != null) {
                String apn = ni.getExtraInfo();
                if(apn != null && apn.equalsIgnoreCase("cmwap")) {
                    return true;
                }
            }
        }
        return false;
    }
	
}
